mod database;
mod diesel_dao;
mod models;
mod mutation;
mod query;
mod schema;
mod subscription;

use std::{net::SocketAddr, sync::Arc};

use axum::{
    response::Html,
    routing::{get, on, MethodFilter},
    Extension, Router,
};
use juniper::RootNode;
use juniper_axum::{graphiql, graphql, playground, subscriptions};
use juniper_graphql_ws::ConnectionConfig;
use mutation::Mutation;
use query::Query;
use subscription::Subscription;
use tokio::net::TcpListener;

use crate::database::Database;

type Schema = RootNode<'static, Query, Mutation, Subscription>;

async fn homepage() -> Html<&'static str> {
    "<html><h1>juniper_axum/simple example</h1>\
           <div>visit <a href=\"/graphiql\">GraphiQL</a></div>\
           <div>visit <a href=\"/playground\">GraphQL Playground</a></div>\
    </html>"
        .into()
}

#[tokio::main]
async fn main() {
    tracing_subscriber::fmt()
        .with_max_level(tracing::Level::INFO)
        .init();

    let database = Database::new();
    let schema = Schema::new(Query, Mutation, Subscription);

    let app = Router::new()
        .route(
            "/graphql",
            on(
                MethodFilter::GET.or(MethodFilter::POST),
                graphql::<Arc<Schema>>,
            ),
        )
        .route(
            "/subscriptions",
            get(subscriptions::ws::<Arc<Schema>>(ConnectionConfig::new(
                database,
            ))),
        )
        .route("/graphiql", get(graphiql("/graphql", "/subscriptions")))
        .route("/playground", get(playground("/graphql", "/subscriptions")))
        .route("/", get(homepage))
        .layer(Extension(Arc::new(schema)));

    let addr = SocketAddr::from(([127, 0, 0, 1], 8080));
    let listener = TcpListener::bind(addr)
        .await
        .unwrap_or_else(|e| panic!("failed to listen on {addr}: {e}"));

    tracing::info!("listening on {addr}");

    axum::serve(listener, app)
        .await
        .unwrap_or_else(|e| panic!("failed to run `axum::serve`: {e}"));
}
